package oracle.forms.fd;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.geom.RoundRectangle2D;
import oracle.forms.handler.IHandler;
import oracle.forms.properties.ID;
import oracle.forms.ui.VTextField;
import sun.misc.resources.Messages;

/**
 * A PJC to conceal part of a TextField content
 * 1.0
 * @author   Francois Degrelle
 * december 2011
 */

public class ConcealTextField extends VTextField implements KeyListener
{
  public final static ID setBounds  = ID.registerProperty("SET_BOUNDS");  
  public final static ID setLog     = ID.registerProperty("SET_LOG");  
  
  private String  original=null;
  private String  crypted=null;
  private VTextField     VTF ;
  private int   deb = -1;
  private int   fin = -1;
  private int   current=-1;
  private char  cChar;
  private Color bgColor = null;
  private Color fgColor = null;
  private boolean bLog = false ;
  
  /*
  public ConcealTextField() {
    this.setBackground(null);
    this.setFill(null);
  }
  */
  
    public void init(IHandler handler)
    {
      super.init(handler);
      // add key listener
      addKeyListener(this);
      VTF = this ;
    }    

  public boolean setProperty(ID property, Object value)
  {

    if (property == setBounds){
      String s = value.toString();
      String s1=null,s2=null;
      int i1, i2;
      int iPos = s.indexOf(",");
      if(iPos > -1) {
          try {
              s1 = s.substring(0,iPos) ;
              s2 = s.substring(iPos+1) ;
              i1 = Integer.parseInt(s1);
              i2 = Integer.parseInt(s2);
              if(i1 <= 0) {
                System.out.println("setConfig() start must be greater than 0");
                return false;
              }
              if(i2 <= 0) {
                System.out.println("setConfig() end must be greater than 0");
                return false;
              }
              if(i2 <= i1) {
                System.out.println("setConfig() end must be greater or equal than start");
                return false;
              }                            
              deb = i1-1;
              fin = i2-1;
              crypted = getCrypted();
              VTF.setText(crypted);
          }
          catch(Exception ex){
          System.out.println("setConfig() incorrect values: "+s);}
      }
      
      return true;
    }
    else if (property == ID.VALUE){
      fgColor = this.getForeground();
      bgColor = this.getBackground();
      super.setProperty(property, value);
      original = value.toString();
      crypted = getCrypted();
      VTF.setText(crypted);
      current = original.length() + 1;
      return true;
    } 
    else if (property == setLog){
      String s = value.toString();
      if(s.equalsIgnoreCase("true")) bLog = true;
      else bLog = false;
      return true;
    }      
    
    else return super.setProperty(property, value);
  }
  
  public Object getProperty(ID property)
  {
    // get the clear value
    if (property == ID.VALUE) {  
      return original ;
    }
    else // default behaviour
    {
      return super.getProperty(property);
    }
  }
       
  
    //
    // Handle the key typed event from the text field.
    //
    public void keyTyped(KeyEvent e) {
    }
        
    //
    // Handle the key pressed event from the text field. 
    //
    public void keyPressed(KeyEvent e) {
      int iCode = e.getKeyCode();
      current = VTF.getCaretPosition();
      VTF.setForeground(bgColor);
      VTF.setText(original);
      try{
        VTF.setCaretPosition(current);
      }
      catch (Exception ex){ex.printStackTrace();}
    }

    //
    // Handle the key released event from the text field. 
    //
    public void keyReleased(KeyEvent e) {
      VTF.setForeground(fgColor);
      current = VTF.getCaretPosition();
      crypted = getCrypted();
      VTF.setText(crypted);
      VTF.setCaretPosition(current);
    }
  
  /*
   * encrypt the string
   */
  String getCrypted(){
    StringBuffer sb = new StringBuffer();
    int i =0;
    int f = fin <= (original.length()-1) ? fin : (original.length()-1);
    original = this.getText();
    if(deb+fin == -2) {
      sb.append(original) ;
    }
    else {
      try{
        for(i=0; i< original.length();i++){
          if(i>= deb && i<= fin) sb.append("*");
          else
          sb.append(original.charAt(i));
        }
      }
      catch(Exception ex){}
    }
    log("original("+original+")");
    log("crypted ("+sb.toString()+")");
    return sb.toString();
  }
 
  void log(String s){
    if(bLog){
      System.out.println(this.getDefaultName()+" - "+s);
    }
  }

}